=begin
(c) TIG 2010
Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided that the above
copyright notice appear in all copies.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
___________________________________________________________________________
'clinesaxes.rb' >>> ../Plugins/ folder
___________________________________________________________________________
Usage:

Run the Tool - Plugins Menu 'Clines on Axes at Point'
Pick a Point.
Three Guides [clines] are added at that Point parallel to the three Axes.
The tool repeats until you Esc or activate another tool.

Run the Tool - Plugins Menu 'Clines Normal at Point'
Pick a Point on a Face.
Three Guides [clines] are added at that Point parallel to the three Axes, 
BUT rotated to be 'Normal' to the picked Face.
The tool repeats until you Esc or activate another tool.
You can move the selected 3 clines to any point later...
It works well with the face in a 'WorkPlane' [WorkPlane.rb]...

Alternatively use the Toolbar 'WorkPlane Tools' buttons, if available
[which can be activated from Views > Toolbars]
___________________________________________________________________________
Donations:
by PayPal to info @ revitrev.org
___________________________________________________________________________
Version:
1.0 20100923 First release.
1.1 20100925 Additional tool 'Clines Normal at Point' added.
             Added to WorkPlane Toolbar too.
1.2 20100926 Guides visibility forced 'on'.
___________________________________________________________________________
=end
###
require 'sketchup.rb'
###
class ClinesAxes

def activate()
  @ip=Sketchup::InputPoint.new
  @ip1=Sketchup::InputPoint.new
  @msg="Clines on Axes: Pick Point..."
  Sketchup::set_status_text(@msg)
  @model=Sketchup.active_model
  @ents=@model.active_entities
  ### force guides 'on'
  @model.rendering_options["HideConstructionGeometry"]=false
  ###
  @view=@model.active_view
  if @view
    @view.lock_inference
    @view.invalidate
  end#if
end

def onMouseMove(flags, x, y, view)
  @ip.pick(view, x, y)
  view.tooltip=@ip.tooltip
  if @ip != @ip1
    view.invalidate if @ip.display? or @ip1.display?
    @ip1.copy!(@ip)
  end#if
  Sketchup::set_status_text(@msg)
end

def draw(view)
  @ip1.draw(view) if @ip.display? or @ip1.display?
end

def onLButtonDown(flags, x, y, view)
  @point=@ip1.position
  self.add_clines()
end

def resume(view)
  view.invalidate if view
  Sketchup::set_status_text(@msg)
end

def deactivate(view)
  view.invalidate if view
  return nil
end
 
def onCancel(flag, view)
  view.invalidate if view
  Sketchup.send_action("selectSelectionTool:")
  return nil
end

def add_clines()
  @model.start_operation("Clines on Axes at Point")
    @ents.add_cline(@point,X_AXIS)
    @ents.add_cline(@point,Y_AXIS)
    @ents.add_cline(@point,Z_AXIS)
  @model.commit_operation
end

end#class
###
class ClinesNormal

def activate()
  @ip=Sketchup::InputPoint.new
  @ip1=Sketchup::InputPoint.new
  @msg="Clines Normal at Point: Pick a Point on a Face or WorkPlane to set the Normal..."
  Sketchup::set_status_text(@msg)
  @model=Sketchup.active_model
  @ents=@model.active_entities
  ### force guides 'on'
  @model.rendering_options["HideConstructionGeometry"]=false
  ###
  @view=@model.active_view
  if @view
    @view.lock_inference
    @view.invalidate
  end#if
  @face=nil
end

def onMouseMove(flags, x, y, view)
  @ip.pick(view, x, y)
  view.tooltip=@ip.tooltip
  if @ip != @ip1
    view.invalidate if @ip.display? or @ip1.display?
    @ip1.copy!(@ip)
  end#if
  ph=view.pick_helper
  ph.do_pick(x,y)
  @face=ph.picked_face
  Sketchup::set_status_text(@msg)
end

def draw(view)
  @ip1.draw(view) if @ip.display? or @ip1.display?
end

def onLButtonDown(flags, x, y, view)
  ph=view.pick_helper
  ph.do_pick(x,y)
  @face=ph.picked_face
  @point=@ip1.position
  if @face
    self.add_clines()
  else
    UI.beep
    puts "Pick a Point on a Face or WorkPlane!"
  end#if
end

def resume(view)
  view.invalidate if view
  Sketchup::set_status_text(@msg)
end

def deactivate(view)
  view.invalidate if view
  return nil
end
 
def onCancel(flag, view)
  view.invalidate if view
  Sketchup.send_action("selectSelectionTool:")
  return nil
end

def add_clines()
  @model.start_operation("Clines Normal at Point")
    gp=@ents.add_group()
    gents=gp.entities
    gents.add_cline(@point,X_AXIS)
    gents.add_cline(@point,Y_AXIS)
    gents.add_cline(@point,Z_AXIS)
    norm=@face.normal
    if not norm==X_AXIS and not norm.reverse==X_AXIS and not norm==Y_AXIS and not norm.reverse==Y_AXIS and not norm==Z_AXIS and not norm.reverse==Z_AXIS
      perp=Z_AXIS.cross(norm)
      ang=Z_AXIS.angle_between(norm)
      tr=Geom::Transformation.rotation(@point,perp,ang)
      gp.transform!(tr)
    end#if
    gp.explode
  @model.commit_operation
end

end#class
###
def clinesaxes()
  Sketchup.active_model.select_tool(ClinesAxes.new)
end
###
def clinesnormal()
  Sketchup.active_model.select_tool(ClinesNormal.new)
end
### menu
if not file_loaded?(File.basename(__FILE__))
  ### toolbar
  text11="Clines Axes"
  text12="Clines Axes - adds Axial clines at a picked point"+"..."
  cmd1=UI::Command.new(text11){Sketchup.active_model.select_tool(ClinesAxes.new)}
  cmd1.tooltip=text11
  cmd1.status_bar_text=text12
  text21 = "Clines Normal"
  text22 = "Clines Normal - adds Normal clines at a picked point"+"..."
  cmd2=UI::Command.new(text21){Sketchup.active_model.select_tool(ClinesNormal.new)}
  cmd2.tooltip=text21
  cmd2.status_bar_text=text22
  if $workPlaneToolbar
    cmd1.small_icon=File.join(File.dirname(__FILE__), "WorkPlane/ClinesAxes16x16.png")
    cmd1.large_icon=File.join(File.dirname(__FILE__), "WorkPlane/ClinesAxes24x24.png")
    $workPlaneToolbar.add_item(cmd1)
    cmd2.small_icon=File.join(File.dirname(__FILE__), "WorkPlane/ClinesNormal16x16.png")
    cmd2.large_icon=File.join(File.dirname(__FILE__), "WorkPlane/ClinesNormal24x24.png")
    $workPlaneToolbar.add_item(cmd2)
    $workPlaneToolbar.restore if $workPlaneToolbar.get_last_state==TB_VISIBLE
  end#if
  ### menu
  UI.menu("Plugins").add_item("Clines on Axes at Point"){clinesaxes()}
  UI.menu("Plugins").add_item("Clines Normal at Point"){clinesnormal()}
end#if
file_loaded(File.basename(__FILE__))
###
